//-----------------------------------------------------------------------
//	File:		$RCSfile: Lab4Part1.V,v $
//	Version:	$Revision: 1.1 $
//	Desc:		Lab4 Part1 - Debugging in Simulation Modules
//	Author:		Greg Gibeling
//	Copyright:	Copyright 2003-2005 UC Berkeley
//	This copyright header must appear in all derivative works.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Section:	Change Log
//-----------------------------------------------------------------------
//	$Log: Lab4Part1.V,v $
//	Revision 1.1  2005/01/31 01:37:58  gdgib
//	Initial Import
//	Copied from EECS150Fall2004 Lab4
//	
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
//	Module:		Lab4PeakDetector
//	Desc:		Outputs the largest of all the inputs given to it
//			so far on the Peak output.
//-----------------------------------------------------------------------
module Lab4PeakDetector(In, Peak, Clock, Reset);
	//---------------------------------------------------------------
	//	Input and Peak Output
	//---------------------------------------------------------------
	input	[3:0]		In;
	output			Peak;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Syetem Inputs
	//---------------------------------------------------------------
	input			Clock, Reset;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Internal Wires and Regs
	//---------------------------------------------------------------
	wire			GreaterEqual;

	reg			Peak;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Instantiate the Comparator
	//---------------------------------------------------------------
	Lab4Comp4	Comp(	.A(		Peak),
				.B(		In),
				.GreaterEqual(	GreaterEqual));
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Peak Register
	//---------------------------------------------------------------
	always @ (posedge Clock) begin
		if (Reset) Peak <=			4'h0;
		else if (GreaterEqual) Peak <=		In;
	end
	//---------------------------------------------------------------
endmodule
//-----------------------------------------------------------------------


//-----------------------------------------------------------------------
//	Module:		Lab4Comp4
//	Desc:		Four bit wide comparator.
//			GreaterEqual =	A <= B
//-----------------------------------------------------------------------
module Lab4Comp4(A, B, GreaterEqual);
	//---------------------------------------------------------------
	//	Inputs to Compare
	//---------------------------------------------------------------
	input	[3:0]		A, B;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Output
	//---------------------------------------------------------------
	output			GreaterEqual;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Internal Cascade Busses
	//---------------------------------------------------------------
	wire	[3:0]		Greater;
	wire	[3:0]		Equal;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Output Generation
	//---------------------------------------------------------------
	assign			GreaterEqual =		Greater[0] | Equal[0];
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Inidividual 1bit Comparators
	//---------------------------------------------------------------
	Lab4Comp1	Bit3(.GreaterIn(      1'b1), .EqualIn(    1'b1), .A(A[3]), .B(B[3]), .GreaterOut(Greater[3]), .EqualOut(Equal[3]));
	Lab4Comp1	Bit2(.GreaterIn(Greater[3]), .EqualIn(Equal[3]), .A(A[2]), .B(B[2]), .GreaterOut(Greater[2]), .EqualOut(Equal[2]));
	Lab4Comp1	Bit1(.GreaterIn(Greater[2]), .EqualIn(Equal[2]), .A(A[1]), .B(B[1]), .GreaterOut(Greater[1]), .EqualOut(Equal[1]));
	Lab4Comp1	Bit0(.GreaterIn(Greater[1]), .EqualIn(Equal[1]), .A(A[0]), .B(B[0]), .GreaterOut(Greater[0]), .EqualOut(Equal[0]));
	//---------------------------------------------------------------
endmodule
//-----------------------------------------------------------------------


//-----------------------------------------------------------------------
//	Module:		Lab4Comp1
//	Desc:		One bit wide, cascadable comparator.
//			GreaterOut =	A < B
//			EqualOut =	A == B
//-----------------------------------------------------------------------
module Lab4Comp1(GreaterIn, EqualIn, A, B, GreaterOut, EqualOut);
	//---------------------------------------------------------------
	//	Cascade Inputs (From Higher Bit)
	//---------------------------------------------------------------
	input			GreaterIn, EqualIn;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Data Inputs
	//---------------------------------------------------------------
	input			A, B;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Cascade Outputs (To Lower Bit)
	//---------------------------------------------------------------
	output			GreaterOut, EqualOut;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Reg Declarations
	//---------------------------------------------------------------
	reg			GreaterOut, EqualOut;
	//---------------------------------------------------------------

	//---------------------------------------------------------------
	//	Combinational Logic
	//---------------------------------------------------------------
	always @ (GreaterIn or EqualIn or A or B) begin
		if (GreaterIn) begin
			GreaterOut =			1'b1;
			EqualOut =			1'b0;
		end
		else if (~EqualIn) begin
			GreaterOut =			1'b0;
			EqualOut =			1'b0;
		end
		else begin
			GreaterOut =			A & B;
			EqualOut =			~(A ^ B);
		end
	end
	//---------------------------------------------------------------
endmodule

